# combinat::partitions(n) returns the number of integer partitions of n #
combinat::partitions:=proc(n) local k;
begin
   # the following avoids stacks problems under MACOS #
   for k from n mod 25 to n step 25 do combinat::partitions2(k) end_for
end_proc:

# the following procedure uses Euler's pentagonal formula

  p(n) + sum((-1)^k*(p(n-w(k))+p(n-w(-k))),k=1..infinity) = 0

  where w(k)=(3*k^2+k)/2. The asymptotic expansion of p(n) is:

                             1/2    /    / 2 n \1/2 \
                            3    exp| PI | --- |    |
                                    \    \  3  /    /
                     p(n) ~ -------------------------
                                      12 n

  Reference: G. Andrews, The Theory of Partitions, Addison-Wesley, 1976.
#
combinat::partitions2:=proc(n) local s,t,k;
option remember;
begin
       s := 0;
       for k from 1 to n do
            t := iquo(3*k^2-k,2);
            if n < t then return(abs(s)) end_if;
            s := combinat::partitions2(n-t)-s;
            t := t+k;
            if n < t then return(abs(s)) end_if;
            s := combinat::partitions2(n-t)+s
        end_for;
end_proc:
combinat::partitions2(0):=1:
